
/*  Various programs concerning the El-Fish .fsh file format.
    Copyright (C) 2004 Vidar Holen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


package holen.vidar.elfish;


/*
Program for viewing random sequences of bytes as an image
of varying width. For my elfish .fsh hacking project.

copyleft Vidar Holen, vidar@vidarholen.net
*/


import java.awt.*;
import java.io.*;
import java.awt.event.*;

public class DistortView extends Frame implements KeyListener, MouseListener {
	int push=0;
	int width=80;
	int multiplier=8;
	char[] b;
        java.awt.Color[] palette;

	

	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {
		System.out.println(e.getModifiers());
		int x=e.getX();
		int y=e.getY();
		int px;
		x=x/multiplier;
		y=y/multiplier;
		px=y*width+x-push;
		if((e.getModifiers()&e.BUTTON2_MASK)!=0) 
			System.out.println("Pixel: #"+px+", 0x"+Integer.toString(px,16)+" ("+(int)b[px]+", 0x"+Integer.toString((int)b[px],16)+")");

		 else if((e.getModifiers()&e.BUTTON1_MASK)!=0) {
				 for(int i=b.length-1; i>px; i--) 
					 b[i]=b[i-1];
				 b[px]=(char)6;
				 System.out.println("Putting");
			 } 
		else if((e.getModifiers()&e.BUTTON3_MASK)!=0) {
				for(int i=px; i<b.length-1; i++) 
					b[i]=b[i+1];
				 b[b.length-1]=(char)6;
				 System.out.println("Popping");
		}
		 repaint();
	}
	

	public void keyTyped(KeyEvent e) {}
	public void keyReleased(KeyEvent e) {}
	public void keyPressed(KeyEvent e) {
		if(e.getKeyCode()==e.VK_UP) width++; else
		if(e.getKeyCode()==e.VK_DOWN) width--; else
		if(e.getKeyCode()==e.VK_RIGHT) push=(push+1)%width; else
		if(e.getKeyCode()==e.VK_LEFT) push=(push-1)%width; else
		if(e.getKeyCode()==e.VK_A) multiplier*=2; else
		if(e.getKeyCode()==e.VK_Z) if(multiplier>1) multiplier/=2;
	       repaint();
	}

	public DistortView(char[] c, String cp) throws Exception {
		b=c;
                loadPalette(cp);
		setSize(640,480);
		setVisible(true);
		addKeyListener(this);
		addMouseListener(this);
	}

        public void loadPalette(String file) throws Exception{
                        palette=new java.awt.Color[256];
                        BufferedReader in=new BufferedReader(new
FileReader(file));
                        String s;
                        while((s=in.readLine())!=null) {
                                int div=s.indexOf(";");
                                int idx=Integer.parseInt(s.substring(0,div));
                                int val=Integer.parseInt(s.substring(div+1),16);
                                palette[idx]=new java.awt.Color(val);
                        }

        }


	public void paint(Graphics g) {
		System.out.println(width+" width, "+push+" push");
		for(int i=0; i<b.length; i++) {
			//g.setColor(new java.awt.Color((int)b[i]<<8));
                        g.setColor(palette[(int)b[i]]);
			g.fillRect((i+push)%width*multiplier,(i+push)/width*multiplier,multiplier,multiplier);
			}
	}
	public static void main(String[] args) throws Exception {
		DataInputStream dis=new DataInputStream(System.in);
		char[] b=new char[10000];
		for(int i=0; i<b.length; i++) {
			b[i]=(char)dis.read();
			if(false && b[i]==0x00) {
				int n=dis.read(); 
				//b[i++]=(char)n;
				if(dis.read()==0xFF)
					n=0xC0-n;
				System.out.println(n+" fill");
			//	if(n<70)
					for(int j=0; j<n && i<10000; j++)
						b[i++]=0;
			}
		}
//		for(int i=0; i<b.length; i++) b[i]=(char)(b[i]==0xAF?b[i]*4:0);
		DistortView dv=new DistortView(b,args[0]);

	}
}


