
/*  Various programs concerning the El-Fish .fsh file format.
    Copyright (C) 2004 Vidar Holen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


package holen.vidar.elfish;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

public class Jelfish extends Canvas implements KeyListener {
    final public static int IDLE=5, PADDLE=4, SWIM=0, HALT=6;
    final public static int START_TRIPPLE=0, MIDDLE_TRIPPLE=1, END_TRIPPLE=2, 
                      START_DOUBLE=5, END_DOUBLE=6, 
                      WAITLEFT=100, WAITRIGHT=101, 
                      NONE=-1;

    int scale=2;
    int delay=400;
    boolean stop=false;
    int x=100,y=100;

    int lastx, lasty, lastw, lasth;
    
    int dirmode=0;
    int nowDir=0, nowAction=IDLE, turnState=NONE, nowTurn=0, frame=0;
    int turn=0, nextDir=-1;
    
    int speed=0;
    boolean mirror=false;
    
    Fish fish;
    FishFrame now;
    ArrayList anim;
    BufferedImage img;


    public Jelfish(String args) throws Exception {
        super();
        fish=Fish.loadFish(args);
        dirmode=fish.getDirections();

        addKeyListener(this);
        //img=new BufferedImage();
        anim=new ArrayList();
        
        new Thread() {
            public void run() {
                while(true) {
                    try {
                        Thread.sleep(delay);
                        update();
                    } catch(Exception e) {}
                }
            }
        }.start();
    }
    
    public void keyPressed(KeyEvent e) {
        boolean m=false;
        if(e.getKeyCode()==e.VK_A) scale++;
        else if(e.getKeyCode()==e.VK_Z) scale--;
        else if(e.getKeyCode()==e.VK_SPACE) stop=!stop;
        else if(e.getKeyCode()==e.VK_U) turn(1);
        else if(e.getKeyCode()==e.VK_I) turn(-1);
        else if(e.getKeyCode()==e.VK_H) turn(2);
        else if(e.getKeyCode()==e.VK_L) turn(-2);
        else if(e.getKeyCode()==e.VK_K) speed(1);
        else if(e.getKeyCode()==e.VK_J) speed(-1);
    }
    public void keyReleased(KeyEvent e) {
    }
    public void keyTyped(KeyEvent e) {
    }

    void turn(int n) {
        nextDir=((nowDir+n)+dirmode)%dirmode;
        turn=n;
        System.out.println(turn);
    }
    void speed(int n) {
        speed+=n;
        if(speed<0) speed=0;
        if(speed>2) speed=2;
    }
    
    public void update() {
        if(stop) return;
        if(frame<anim.size()) {
            now=(FishFrame)anim.get(frame);
            frame++;
        } else {
            System.out.println("-");
            frame=0;
            boolean swim=false;

            if(turnState==NONE) {
                if(turn!=0) {
                    nowTurn=turn;
                    turn=0;
                    if(nowAction==SWIM) {
                        if(nowTurn<0) {
                            if(mirror) 
                                turnState=nowTurn==-1?START_DOUBLE:START_TRIPPLE;
                            else turnState=WAITRIGHT;
                        } else {
                            if(!mirror)
                                turnState=nowTurn==-1?START_DOUBLE:START_TRIPPLE;
                            else turnState=WAITLEFT;
                        }
                    } else {
                        turnState=nowTurn==-1 || nowTurn==1 ? START_DOUBLE:START_TRIPPLE;
                    }
                }
            }

            switch(turnState) {
                case END_DOUBLE:
                case END_TRIPPLE:
                    turnState=NONE;
                    swim=true;
                    break;

                case START_TRIPPLE:
                    turnState=MIDDLE_TRIPPLE;
                    if(mirror) nowDir--; else nowDir++;
                    nowDir=(nowDir+dirmode)%dirmode;
                    nowAction=2;
                    break;

                case START_DOUBLE: 
                    turnState=END_DOUBLE;
                    if(mirror) nowDir--; else nowDir++;
                    nowDir=(nowDir+dirmode)%dirmode;
                    nowAction=3;
                    break;

                case MIDDLE_TRIPPLE: 
                    turnState=END_TRIPPLE;
                    if(mirror) nowDir--; else nowDir++;
                    nowDir=(nowDir+dirmode)%dirmode;
                    nowAction=3;
                    break;
                    
                case WAITLEFT:
                    if(nowAction!=SWIM || !mirror) {
                        nowAction=1;
                        if(turn==2) turnState=START_TRIPPLE;
                        else if(turn==1) turnState=START_DOUBLE;
                    } else swim=true;
                    break;
                case WAITRIGHT:
                    if(nowAction!=SWIM || mirror) { 
                        nowAction=1;
                        if(turn==-2) turnState=START_TRIPPLE;
                        else if(turn==-1) turnState=START_DOUBLE;
                    } else swim=true;
                    break;
                default:
                    swim=true;
            }

            if(swim) {
                switch(nowAction) {
                    case SWIM:
                        if(speed<=1) {
                            nowAction=HALT;
                        } else {
                            mirror=!mirror;
                        }
                        break;
                    case PADDLE:
                        mirror=!mirror;
                        if(speed==0)
                            nowAction=IDLE;
                        else if(speed==2)
                            nowAction=SWIM;
                        break;
                    case IDLE:
                        mirror=!mirror;
                        if(speed==1)
                            nowAction=PADDLE;
                        else if(speed==2) 
                            nowAction=SWIM;
                        break;
                    case HALT:
                        if(speed==0)
                            nowAction=IDLE;
                        else if(speed==1) 
                            nowAction=PADDLE;
                        else if(speed==2) 
                            nowAction=SWIM;
                        break;
                }
            }
            System.out.println(
                    "turnState="+turnState+
                    ", nowAction="+nowAction+
                    ", speed="+speed+
                    ", turn="+turn+
                    ", frame="+frame);
            anim=fish.getSequence(mirror
                    ?fish.invertDirection(nowDir)
                    :nowDir
                    ,nowAction);
            now=(FishFrame)anim.get(0);
            frame++;
        }
        repaint();
    }
    
    public void paint(Graphics g) {
        int xo=x+now.x+(mirror?now.w*2:0), yo=y+now.y, w=mirror?-now.w:now.w, h=now.h;
        xo*=scale;
        yo*=scale;
        w*=scale;
        h*=scale;
        
//        g.clearRect(0,0,getWidth(),getHeight());
        g.drawImage(now.img,xo,yo,w,h,null);
    }
    public static void main(String[] args) throws Exception {
        Jelfish s=new Jelfish(args[0]);
        
        JFrame f=new JFrame("Fish");
        f.setDefaultCloseOperation(f.EXIT_ON_CLOSE);
        f.getContentPane().setLayout(new GridLayout(1,1));
        f.getContentPane().add(s);
        f.setSize(640,480);
        f.setVisible(true);
    }
}


